from .utils import *
import logging
logger = logging.getLogger(__name__)

def create_snapshot(client, indices='_all', name=None,
                    prefix='curator-', repository='',
                    ignore_unavailable=False, include_global_state=True,
                    partial=False, wait_for_completion=True, request_timeout=21600):
    """
    Create a snapshot of provided indices (or ``_all``) that are open.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to snapshot. Default is ``_all``
    :arg name: What to name the snapshot. `prefix` +
        datestamp if omitted.
    :arg prefix: Override the default with this value. Defaults to
        ``curator-``
    :arg repository: The Elasticsearch snapshot repository to use
    :arg wait_for_completion: Wait (or not) for the operation
        to complete before returning.  (default: `True`)
    :type wait_for_completion: bool
    :arg ignore_unavailable: Ignore unavailable shards/indices.
        (default: `False`)
    :type ignore_unavailable: bool
    :arg include_global_state: Store cluster global state with snapshot.
        (default: `True`)
    :type include_global_state: bool
    :arg partial: Do not fail if primary shard is unavailable. (default:
        `False`)
    :type partial: bool
    :rtype bool:
    """
    # Return True if it is skipped
    if not repository:
        logger.error('Missing required repository parameter')
        return False
    if not indices == '_all':
        indices = prune_closed(client, indices)
    if not indices:
        logger.error("No indices provided.")
        return False
    repo_access = (1, 4, 0)
    version_number = get_version(client)
    if version_number >= repo_access:
        try:
            nodes = client.snapshot.verify_repository(repository=repository)['nodes']
            logger.debug('Nodes with verified repository access: {0}'.format(nodes))
        except Exception:
            logger.error('Failed to verify all nodes have repository access.')
            return False
    body=create_snapshot_body(indices, ignore_unavailable=ignore_unavailable,
                                include_global_state=include_global_state,
                                partial=partial)
    name = name if name else prefix + datetime.utcnow().strftime('%Y%m%d%H%M%S')
    logger.info("Snapshot name: {0}".format(name))
    all_snaps = get_snapshots(client, repository=repository)
    if name in all_snaps:
        logger.error("A snapshot with name '{0}' already exists.".format(name))
        return False
    try:
        client.snapshot.create(repository=repository, snapshot=name, body=body,
                                wait_for_completion=wait_for_completion,
                                request_timeout=request_timeout)
        return True
    except elasticsearch.TransportError:
        logger.error("Client raised a TransportError.")
        return False

def delete_snapshot(client, snapshot=None, repository=None):
    """
    Delete a single snapshot from a given repository by name

    :arg client: The Elasticsearch client connection
    :arg snapshot: The snapshot name
    :arg repository: The Elasticsearch snapshot repository to use
    """
    if not repository:
        logger.error('Missing required repository parameter')
        return False
    if not snapshot:
        logger.error('Missing required snapshot parameter')
        return False
    if check_csv(snapshot):
        logger.error('Cannot delete multiple snapshots at once.  CSV value or list detected: {0}'.format(snapshot))
        return False
    try:
        client.snapshot.delete(repository=repository, snapshot=snapshot)
        return True
    except elasticsearch.RequestError:
        logger.error("Unable to delete snapshot {0} from repository {1}.  Check logs for more information.".format(snapshot, repository))
        return False



from datetime import timedelta, datetime, date
import elasticsearch
import time
import re
import sys
import logging
logger = logging.getLogger(__name__)

def get_alias(client, alias):
    """
    Return information about the specified alias.

    :arg client: The Elasticsearch client connection
    :arg alias: Alias name to operate on.
    :rtype: list of strings
    """
    if client.indices.exists_alias(alias):
        return client.indices.get_alias(name=alias).keys()
    else:
        logger.error('Unable to find alias {0}.'.format(alias))
        return False

def get_indices(client):
    try:
        indices = list(client.indices.get_settings(
            index='*', params={'expand_wildcards': 'open,closed'}))
        logger.debug("All indices: {0}".format(indices))
        return indices
    except Exception:
        logger.error("Failed to get indices.")
        return False

def ensure_list(indices):
    """
    Return a list, even if indices is a single value

    :arg indices: A list of indices to act upon
    :rtype: list
    """
    if type(indices) is not type(list()):   # in case of a single value passed
        indices = [indices]
    return indices

def to_csv(indices):
    """
    Return a csv string from a list of indices, or a single value if only one
    value is present

    :arg indices: A list of indices to act on, or a single value, which could be
        in the format of a csv string already.
    :rtype: str
    """
    indices = ensure_list(indices) # in case of a single value passed
    if indices:
        return ','.join(sorted(indices))
    else:
        return None

def check_csv(value):
    """
    Some of the curator methods should not operate against multiple indices at
    once.  This method can be used to check if a list or csv has been sent.

    :arg value: The value to test, if list or csv string
    :rtype: bool
    """
    if type(value) is type(list()):
        return True
    string = False
    # Python3 hack because it doesn't recognize unicode as a type anymore
    if sys.version_info < (3, 0):
        if type(value) is type(unicode()):
            value = str(value)
    if type(value) is type(str()):
        if len(value.split(',')) > 1: # It's a csv string.
            return True
        else: # There's only one value here, so it's not a csv string
            return False
    else:
        logger.error("Passed value: {0} is not a list or a string but is of type {1}".format(value, type(value)))
        sys.exit(1)

def index_closed(client, index_name):
    """
    Return `True` if the indicated index is closed.

    :arg client: The Elasticsearch client connection
    :arg index_name: The index name
    :rtype: bool
    """
    index_metadata = client.cluster.state(
        index=index_name,
        metric='metadata',
    )
    return index_metadata['metadata']['indices'][index_name]['state'] == 'close'

def get_segmentcount(client, index_name):
    """
    Return a tuple of `(shardcount, segmentcount)` from the provided
    `index_name`.

    :arg client: The Elasticsearch client connection
    :arg index_name: The index name
    :rtype: tuple
    """
    shards = client.indices.segments(index=index_name)['indices'][index_name]['shards']
    segmentcount = 0
    totalshards = 0 # We will increment this manually to capture all replicas...
    for shardnum in shards:
        for shard in range(0,len(shards[shardnum])):
            segmentcount += shards[shardnum][shard]['num_search_segments']
            totalshards += 1
    return totalshards, segmentcount

def get_version(client):
    """
    Return the ES version number as a tuple.
    Omits trailing tags like -dev, or Beta

    :arg client: The Elasticsearch client connection
    :rtype: tuple
    """
    version = client.info()['version']['number']
    version = version.split('-')[0]
    if len(version.split('.')) > 3:
        version = version.split('.')[:-1]
    else:
       version = version.split('.')
    return tuple(map(int, version))

def is_master_node(client):
    """
    Return `True` if the connected client node is the elected master node in
    the Elasticsearch cluster, otherwise return `False`.

    :arg client: The Elasticsearch client connection
    :rtype: bool
    """
    my_node_id = list(client.nodes.info('_local')['nodes'])[0]
    master_node_id = client.cluster.state(metric='master_node')['master_node']
    return my_node_id == master_node_id

def get_repository(client, repository=''):
    """
    Return configuration information for the indicated repository.

    :arg client: The Elasticsearch client connection
    :arg repository: The Elasticsearch snapshot repository to use
    :rtype: dict
    """
    try:
        return client.snapshot.get_repository(repository=repository)
    except (elasticsearch.TransportError, elasticsearch.NotFoundError):
        logger.error("Repository {0} not found.".format(repository))
        return False

def get_snapshot(client, repository='', snapshot=''):
    """
    Return information about a snapshot (or a comma-separated list of snapshots)
    If no snapshot specified, it will return all snapshots.  If none exist, an
    empty dictionary will be returned.

    :arg client: The Elasticsearch client connection
    :arg repository: The Elasticsearch snapshot repository to use
    :arg snapshot: The snapshot name, or a comma-separated list of snapshots
    :rtype: dict
    """
    if not repository:
        logger.error('Missing required repository parameter')
        return False
    try:
        return client.snapshot.get(repository=repository, snapshot=snapshot)
    except (elasticsearch.TransportError, elasticsearch.NotFoundError):
        logger.error("Snapshot: {0} or repository: {1} not found.".format(snapshot, repository))
        return False

def get_snapshots(client, repository=None):
    """
    Get ``_all`` snapshots from repository and return a list.

    :arg client: The Elasticsearch client connection
    :arg repository: The Elasticsearch snapshot repository to use
    :rtype: list of strings
    """
    if not repository:
        logger.error('Missing required repository parameter')
        return False
    try:
        allsnaps = client.snapshot.get(repository=repository, snapshot="_all")['snapshots']
        return [snap['snapshot'] for snap in allsnaps if 'snapshot' in snap.keys()]
    except (elasticsearch.TransportError, elasticsearch.NotFoundError):
        logger.error("Unable to find all snapshots in repository: {0}".format(repository))
        return False

def create_snapshot_body(indices, ignore_unavailable=False,
                         include_global_state=True, partial=False):
    """
    Create the request body for creating a snapshot from the provided
    arguments.

    :arg indices: A single index, or list of indices to snapshot.
    :arg ignore_unavailable: Boolean. Ignore unavailable shards/indices.
        (default: `False`)
    :arg include_global_state: Boolean. Store cluster global state with snapshot.
        (default: `True`)
    :arg partial: Boolean. Do not fail if primary shard is unavailable.
        (default: `False`)
    :rtype: dict
    """
    if not indices:
        logger.error('Missing required repository parameter')
        return False
    body = {
        "ignore_unavailable": ignore_unavailable,
        "include_global_state": include_global_state,
        "partial": partial,
    }
    if indices == '_all':
        body["indices"] = indices
    else:
        body["indices"] = to_csv(indices)
    return body

def prune_kibana(indices):
    """Remove any index named `.kibana`, `kibana-int`, or `.marvel-kibana`

    :arg indices: A list of indices to act upon.
    :rtype: list
    """
    indices = ensure_list(indices)
    if '.marvel-kibana' in indices:
        indices.remove('.marvel-kibana')
    if 'kibana-int' in indices:
        indices.remove('kibana-int')
    if '.kibana' in indices:
        indices.remove('.kibana')
    return indices

def prune_closed(client, indices):
    """
    Return list of indices that are not closed.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :rtype: list
    """
    indices = ensure_list(indices)
    retval = []
    for idx in list(indices):
        if not index_closed(client, idx):
            retval.append(idx)
        else:
            logger.info('Skipping index {0}: Already closed.'.format(idx))
    return sorted(retval)

def prune_allocated(client, indices, key, value):
    """
    Return list of indices that do not have the routing allocation rule of
    `key=value`

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg key: The allocation attribute to check for
    :arg value: The value to check for
    :rtype: list
    """
    indices = ensure_list(indices)
    retval = []
    for idx in indices:
        settings = client.indices.get_settings(
            index=idx,
        )
        try:
            has_routing = settings[idx]['settings']['index']['routing']['allocation']['require'][key] == value
        except KeyError:
            has_routing = False
        if has_routing:
            logger.debug('Skipping index {0}: Already has allocation rule {1} applied.'.format(idx, key + "=" + value))
        else:
            retval.append(idx)
    return sorted(retval)



from .utils import *
import elasticsearch
import time
import logging
logger = logging.getLogger(__name__)

def disable_bloom_filter(client, indices, delay=None):
    """
    Disable the bloom filter cache for the list of indices.
    This method will ignore closed indices.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg delay: Pause *n* seconds after operating on each index
    :rtype: bool
    """
    indices = prune_closed(client, indices)
    no_more_bloom = (1, 4, 0)
    version_number = get_version(client)
    if version_number >= no_more_bloom:
        logger.warn('Bloom filters no longer exist for search in Elasticsearch since v1.4.0')
        return True
    else:
        try:
            if delay:
                if delay > 0:
                    return loop_bloom(client, indices, delay)
            else:
                client.indices.put_settings(index=to_csv(indices),
                    body='index.codec.bloom.load=false')
                return True
        except Exception:
            logger.error("Error disabling bloom filters.  Check logs for more information.")
            return False

def loop_bloom(client, indices, delay):
    """
    Iterate over list of indices.  Only called from within
    :py:func:`curator.api.disable_bloom_filter`

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg delay: Pause *n* seconds after operating on each index
    :rtype: bool
    """
    retval = True
    for i in indices:
        success = disable_bloom_filter(client, i)
        # If fail on even one iteration, we fail period
        if not success:
            retval = False
        time.sleep(delay)
    return retval

def bloom(client, indices, delay=None):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg delay: Pause *n* seconds after operating on each index
    :rtype: bool
    """

    return disable_bloom_filter(client, indices, delay=delay)



from .utils import *

def show(object_list):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg object_list: A list of indices or snapshots to show
    :rtype: bool
    """
    for obj in ensure_list(object_list):
        print('{0}'.format(obj))
    return True



from .utils import *
import elasticsearch
import logging
logger = logging.getLogger(__name__)

def open_indices(client, indices):
    """
    Open the indicated indices.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :rtype: bool
    """
    indices = ensure_list(indices)
    try:
        # Opening an already open index has no effect.
        client.indices.open(index=to_csv(indices))
        return True
    except Exception:
        logger.error("Error opening indices.  Check logs for more information.")
        return False

def opener(client, indices):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :rtype: bool
    """
    return open_indices(client, indices)



import elasticsearch
import click
import re
import sys

from .utils import *
from .. import __version__

import logging
logger = logging.getLogger(__name__)

try:
    from logging import NullHandler
except ImportError:
    from logging import Handler

    class NullHandler(Handler):
        def emit(self, record):
            pass

DEFAULT_ARGS = {
    'host': 'localhost',
    'url_prefix': '',
    'port': 9200,
    'http_auth': None,
    'use_ssl': False,
    'timeout': 30,
    'dry_run': False,
    'debug': False,
    'log_level': 'INFO',
    'logformat': 'default',
}


def get_repository(client, repository=''):
    """
    Return configuration information for the indicated repository.
    :arg client: The Elasticsearch client connection
    :arg repository: The Elasticsearch snapshot repository to use
    :rtype: dict
    """
    try:
        return client.snapshot.get_repository(repository=repository)
    except elasticsearch.NotFoundError:
        logger.debug("Repository {0} not found.".format(repository))
        return None

def show_repos(client):
    for repository in sorted(get_repository(client, '_all').keys()):
        print('{0}'.format(repository))
    sys.exit(0)


def create_repo_body(repo_type=None,
                     compress=True, concurrent_streams=None, chunk_size=None,
                     max_restore_bytes_per_sec=None,
                     max_snapshot_bytes_per_sec=None,
                     location=None,
                     bucket=None, region=None, base_path=None, access_key=None,
                     secret_key=None, **kwargs):
    """Create the request body for creating a repository"""
    # This shouldn't happen, but just in case...
    if not repo_type:
        click.echo(click.style('Missing required parameter --repo_type', fg='red', bold=True))
        sys.exit(1)

    argdict = locals()
    body = {}
    body['type'] = argdict['repo_type']
    body['settings'] = {}
    settings = []
    maybes   = [
                'compress', 'concurrent_streams', 'chunk_size',
                'max_restore_bytes_per_sec', 'max_snapshot_bytes_per_sec'
               ]
    s3       = ['bucket', 'region', 'base_path', 'access_key', 'secret_key']

    settings += [i for i in maybes if argdict[i]]
    # Type 'fs'
    if argdict['repo_type'] == 'fs':
        settings.append('location')
    # Type 's3'
    if argdict['repo_type'] == 's3':
        settings += [i for i in s3 if argdict[i]]
    for k in settings:
        body['settings'][k] = argdict[k]
    return body

def create_repository(client, **kwargs):
    """Create repository with repository and body settings"""
    if not 'repository' in kwargs:
        click.echo(click.style('Missing required parameter --repository', fg='red', bold=True))
        sys.exit(1)
    else:
        repository = kwargs['repository']

    try:
        body = create_repo_body(**kwargs)
        logging.info("Checking if repository {0} already exists...".format(repository))
        result = get_repository(client, repository=repository)
        if not result:
            logging.info("Repository {0} not in Elasticsearch. Continuing...".format(repository))
            client.snapshot.create_repository(repository=repository, body=body)
        elif result is not None and repository not in result:
            logging.info("Repository {0} not in Elasticsearch. Continuing...".format(repository))
            client.snapshot.create_repository(repository=repository, body=body)
        else:
            logger.error("Unable to create repository {0}.  A repository with that name already exists.".format(repository))
            sys.exit(0)
    except Exception as e:
        logger.error("Unable to create repository {0}.  Exception {1}  Check logs for more information.".format(repository, e.message))
        return False
    logger.info("Repository {0} creation initiated...".format(repository))
    test_result = get_repository(client, repository)
    if repository in test_result:
        logger.info("Repository {0} creation validated.".format(repository))
        return True
    else:
        logger.error("Repository {0} failed validation...".format(repository))
        return False

def delete_callback(ctx, param, value):
    if not value:
        ctx.abort()

@click.command(short_help='Filesystem Repository')
@click.option('--repository', required=True, type=str, help='Repository name')
@click.option('--location', required=True, type=str,
            help='Shared file-system location. Must match remote path, & be accessible to all master & data nodes')
@click.option('--compression', type=bool, default=True, show_default=True,
            help='Enable/Disable compression.')
@click.option('--concurrent_streams', type=int, default=5, show_default=True,
            help='Number of streams (per node) performing snapshot.')
@click.option('--chunk_size', type=str,
            help='Chunk size, e.g. 1g, 10m, 5k. [unbounded]')
@click.option('--max_restore_bytes_per_sec', type=str, default='20mb',
            show_default=True,
            help='Throttles per node restore rate (per second).')
@click.option('--max_snapshot_bytes_per_sec', type=str, default='20mb',
            show_default=True,
            help='Throttles per node snapshot rate (per second).')
@click.pass_context
def fs(
    ctx, repository, location, compression, concurrent_streams, chunk_size,
    max_restore_bytes_per_sec, max_snapshot_bytes_per_sec):
    """
    Create a filesystem repository.
    """
    client = get_client(**ctx.parent.params)
    success = create_repository(client, repo_type='fs', **ctx.params)
    if not success:
        sys.exit(1)

@click.command(short_help='S3 Repository')
@click.option('--repository', required=True, type=str, help='Repository name')
@click.option('--bucket', required=True, type=str, help='S3 bucket name')
@click.option('--region', type=str, help='S3 region. [US Standard]')
@click.option('--base_path', type=str, help='S3 base path. [root]')
@click.option('--access_key', type=str,
            help='S3 access key. [value of cloud.aws.access_key]')
@click.option('--secret_key', type=str,
            help='S3 secret key. [value of cloud.aws.secret_key]')
@click.option('--compression', type=bool, default=True, show_default=True,
            help='Enable/Disable compression.')
@click.option('--concurrent_streams', type=int, default=5, show_default=True,
            help='Number of streams (per node) performing snapshot.')
@click.option('--chunk_size', type=str,
            help='Chunk size, e.g. 1g, 10m, 5k. [unbounded]')
@click.option('--max_restore_bytes_per_sec', type=str, default='20mb',
            show_default=True,
            help='Throttles per node restore rate (per second).')
@click.option('--max_snapshot_bytes_per_sec', type=str, default='20mb',
            show_default=True,
            help='Throttles per node snapshot rate (per second).')
@click.pass_context
def s3(
    ctx, repository, bucket, region, base_path, access_key, secret_key,
    compression, concurrent_streams, chunk_size, max_restore_bytes_per_sec,
    max_snapshot_bytes_per_sec):
    """
    Create an S3 repository.
    """
    client = get_client(**ctx.parent.params)
    success = create_repository(client, repo_type='s3', **ctx.params)
    if not success:
        sys.exit(1)

@click.group()
@click.option('--host', help='Elasticsearch host.', default=DEFAULT_ARGS['host'])
@click.option('--url_prefix', help='Elasticsearch http url prefix.', default=DEFAULT_ARGS['url_prefix'])
@click.option('--port', help='Elasticsearch port.', default=DEFAULT_ARGS['port'], type=int)
@click.option('--use_ssl', help='Connect to Elasticsearch through SSL.', is_flag=True, default=DEFAULT_ARGS['use_ssl'])
@click.option('--http_auth', help='Use Basic Authentication ex: user:pass', default=DEFAULT_ARGS['http_auth'])
@click.option('--timeout', help='Connection timeout in seconds.', default=DEFAULT_ARGS['timeout'], type=int)
@click.option('--master-only', is_flag=True, help='Only operate on elected master node.')
@click.option('--debug', is_flag=True, help='Debug mode', default=DEFAULT_ARGS['debug'])
@click.option('--loglevel', help='Log level', default=DEFAULT_ARGS['log_level'])
@click.option('--logfile', help='log file')
@click.option('--logformat', help='Log output format [default|logstash].', default=DEFAULT_ARGS['logformat'])
@click.version_option(version=__version__)
@click.pass_context
def repomgrcli(ctx, host, url_prefix, port, use_ssl, http_auth, timeout, master_only, debug, loglevel, logfile, logformat):
    """Repository manager for Elasticsearch Curator.
    """

    # Setup logging
    if debug:
        numeric_log_level = logging.DEBUG
        format_string = '%(asctime)s %(levelname)-9s %(name)22s %(funcName)22s:%(lineno)-4d %(message)s'
    else:
        numeric_log_level = getattr(logging, loglevel.upper(), None)
        format_string = '%(asctime)s %(levelname)-9s %(message)s'
        if not isinstance(numeric_log_level, int):
            raise ValueError('Invalid log level: {0}'.format(loglevel))

    handler = logging.StreamHandler(
        open(logfile, 'a') if logfile else sys.stderr)
    if logformat == 'logstash':
        handler.setFormatter(LogstashFormatter())
    else:
        handler.setFormatter(logging.Formatter(format_string))
    logging.root.addHandler(handler)
    logging.root.setLevel(numeric_log_level)

    # Filter out logging from Elasticsearch and associated modules by default
    if not debug:
        for handler in logging.root.handlers:
            handler.addFilter(
                Whitelist(
                    'root', '__main__', 'curator', 'curator.curator',
                    'curator.api', 'curator.cli', 'curator.api',
                    'curator.cli'
                )
            )

    # Setting up NullHandler to handle nested elasticsearch.trace Logger
    # instance in elasticsearch python client
    logging.getLogger('elasticsearch.trace').addHandler(NullHandler())

@repomgrcli.group('create')
@click.pass_context
def _create(ctx):
    """Create an Elasticsearch repository"""
_create.add_command(fs)
_create.add_command(s3)

@repomgrcli.command('show')
@click.pass_context
def show(ctx):
    """
    Show all repositories
    """
    client = get_client(**ctx.parent.params)
    if not isinstance(client, elasticsearch.client.Elasticsearch):
        click.echo('{0}'.format(ctx.get_help()))
        click.echo(click.style('ERROR. Unable to establish connection to Elasticsearch.  Please check your settings.', fg='red', bold=True))
        sys.exit(1)
    show_repos(client)

@repomgrcli.command('delete')
@click.option('--repository', required=True, help='Repository name', type=str)
@click.option('--yes', is_flag=True, callback=delete_callback, expose_value=False,
                prompt='Are you sure you want to delete the repository?')
@click.pass_context
def _delete(ctx, repository):
    """Delete an Elasticsearch repository"""
    client = get_client(**ctx.parent.params)
    try:
        logger.info('Deleting repository {0}...'.format(repository))
        client.snapshot.delete_repository(repository=repository)
        sys.exit(0)
    except elasticsearch.NotFoundError as e:
        logger.error("Unable to delete repository: {0}  Exception: {1}".format(repository, e.message))
        sys.exit(1)

def main():
    repomgrcli()

if __name__ == '__main__':
     main()



import click
from .index_selection import *

import logging
logger = logging.getLogger(__name__)

@cli.group('open')
@click.pass_context
def _open(ctx):
    """Open indices"""
_open.add_command(indices)



import click
from .index_selection import *
from .snapshot_selection import *

import logging
logger = logging.getLogger(__name__)

@cli.group('delete')
@click.option('--disk-space', type=float, expose_value=True,
            help='Delete indices beyond DISK_SPACE gigabytes.')
@click.option('--reverse', type=bool, default=True, expose_value=True,
            show_default=True, is_eager=True,
            help='Only valid with --disk-space. Affects sort order of the indices.  True means reverse-alphabetical (if dates are involved, older is deleted first).')
@click.pass_context
def delete(ctx, disk_space, reverse):
    """Delete indices or snapshots"""
delete.add_command(indices)
delete.add_command(snapshots)



from .utils import *
import elasticsearch
import time
import logging
logger = logging.getLogger(__name__)

def optimize_index(client, index_name, max_num_segments=None,
                request_timeout=21600):
    """
    Optimize (Lucene forceMerge) index to `max_num_segments` per shard

    :arg client: The Elasticsearch client connection
    :arg index_name: The index name
    :arg max_num_segments: Merge to this number of segments per shard.
    :rtype: bool
    """
    if not max_num_segments:
        logger.error("Mising value for max_num_segments.")
        return False
    if check_csv(index_name):
        logger.error("Must specify only a single index as an argument.")
        return False
    if index_closed(client, index_name): # Don't try to optimize a closed index
        logger.info('Skipping index {0}: Already closed.'.format(index_name))
        return True
    else:
        shards, segmentcount = get_segmentcount(client, index_name)
        logger.debug('Index {0} has {1} shards and {2} segments total.'.format(index_name, shards, segmentcount))
        if segmentcount > (shards * max_num_segments):
            logger.info('Optimizing index {0} to {1} segments per shard.  Please wait...'.format(index_name, max_num_segments))
            try:
                client.indices.optimize(index=index_name, max_num_segments=max_num_segments, request_timeout=request_timeout)
                return True
            except Exception:
                logger.error("Error optimizing index {0}.  Check logs for more information.".format(index_name))
                return False
        else:
            logger.info('Skipping index {0}: Already optimized.'.format(index_name))
            return True

def optimize(client, indices, max_num_segments=None, delay=0, request_timeout=21600):
    """
    Helper method called by the CLI.

    :arg client: The Elasticsearch client connection
    :arg indices: A list of indices to act on
    :arg max_num_segments: Merge to this number of segments per shard.
    :rtype: bool
    """
    retval = True
    for i in ensure_list(indices):
        # If we fail once, we fail completely
        success = optimize_index(client, i, max_num_segments=max_num_segments, request_timeout=request_timeout)
        if not success:
            retval = False
        time.sleep(delay)
    return retval



import elasticsearch
import click
import re
from .utils import *
from .. import __version__

import logging
logger = logging.getLogger(__name__)

try:
    from logging import NullHandler
except ImportError:
    from logging import Handler

    class NullHandler(Handler):
        def emit(self, record):
            pass

DEFAULT_ARGS = {
    'host': 'localhost',
    'url_prefix': '',
    'port': 9200,
    'http_auth': None,
    'use_ssl': False,
    'timeout': 30,
    'dry_run': False,
    'debug': False,
    'log_level': 'INFO',
    'logformat': 'default',
}

@click.group()
@click.option('--host', help='Elasticsearch host.', default=DEFAULT_ARGS['host'])
@click.option('--url_prefix', help='Elasticsearch http url prefix.', default=DEFAULT_ARGS['url_prefix'])
@click.option('--port', help='Elasticsearch port.', default=DEFAULT_ARGS['port'], type=int)
@click.option('--use_ssl', help='Connect to Elasticsearch through SSL.', is_flag=True, default=DEFAULT_ARGS['use_ssl'])
@click.option('--http_auth', help='Use Basic Authentication ex: user:pass', default=DEFAULT_ARGS['http_auth'])
@click.option('--timeout', help='Connection timeout in seconds.', default=DEFAULT_ARGS['timeout'], type=int)
@click.option('--master-only', is_flag=True, help='Only operate on elected master node.')
@click.option('--dry-run', is_flag=True, help='Do not perform any changes.', default=DEFAULT_ARGS['dry_run'])
@click.option('--debug', is_flag=True, help='Debug mode', default=DEFAULT_ARGS['debug'])
@click.option('--loglevel', help='Log level', default=DEFAULT_ARGS['log_level'])
@click.option('--logfile', help='log file')
@click.option('--logformat', help='Log output format [default|logstash].', default=DEFAULT_ARGS['logformat'])
@click.version_option(version=__version__)
@click.pass_context
def cli(ctx, host, url_prefix, port, use_ssl, http_auth, timeout, master_only, dry_run, debug, loglevel, logfile, logformat):
    """Curator for Elasticsearch indices. See http://github.com/elastic/curator/wiki
    """

    # Setup logging
    if debug:
        numeric_log_level = logging.DEBUG
        format_string = '%(asctime)s %(levelname)-9s %(name)22s %(funcName)22s:%(lineno)-4d %(message)s'
    else:
        numeric_log_level = getattr(logging, loglevel.upper(), None)
        format_string = '%(asctime)s %(levelname)-9s %(message)s'
        if not isinstance(numeric_log_level, int):
            raise ValueError('Invalid log level: {0}'.format(loglevel))

    handler = logging.StreamHandler(
        open(logfile, 'a') if logfile else sys.stderr)
    if logformat == 'logstash':
        handler.setFormatter(LogstashFormatter())
    else:
        handler.setFormatter(logging.Formatter(format_string))
    logging.root.addHandler(handler)
    logging.root.setLevel(numeric_log_level)

    # Filter out logging from Elasticsearch and associated modules by default
    if not debug:
        for handler in logging.root.handlers:
            handler.addFilter(
                Whitelist(
                    'root', '__main__', 'curator', 'curator.curator',
                    'curator.api', 'curator.cli', 'curator.api',
                    'curator.cli'
                )
            )

    # Setting up NullHandler to handle nested elasticsearch.trace Logger
    # instance in elasticsearch python client
    logging.getLogger('elasticsearch.trace').addHandler(NullHandler())
